@@ -108,18 +108,19 @@ forecast.lm <- function(object, newdata, h=10, level=c(80,95), fan=FALSE, lambda
       newdata <- data.frame(trend,season)
   }
   newdata <- as.data.frame(newdata)
+  # If only one column, assume its name.
+  if(ncol(newdata)==1 & colnames(newdata)[1]=="newdata")
+    colnames(newdata) <- as.character(formula(object$model))[3]
 
   # Check regressors included in newdata.
-  xreg <- attributes(terms(object$model))$term.labels
-  if(any(!is.element(xreg,colnames(newdata))))
-    stop("Predictor variables not included")
+  # Not working so removed for now.
+  #xreg <- attributes(terms(object$model))$term.labels
+  #if(any(!is.element(xreg,colnames(newdata))))
+  #  stop("Predictor variables not included")
 
   responsevar <- as.character(formula(object$model))[2]
   responsevar <- gsub("`","",responsevar)
   object$x <- model.frame(object$model)[,responsevar]
-  # If only one column, assume its name.
-  if(ncol(newdata)==1 & colnames(newdata)[1]=="newdata")
-    colnames(newdata) <- as.character(formula(object$model))[3]
 
   out <- list()
   nl <- length(level)
